// -*- c++ -*- 
//
// BeMailAccount
// Copyright (c) 2001, Okada Jun
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
// 
// 1. Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
// 
// 3. The name of author may not be used to endorse or promote products
// derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 


#include <cstdio>
#include <Application.h>
#include <Roster.h>
#include <File.h>
#include <AppFileInfo.h>
#include <Bitmap.h>
#include <Looper.h>
#include <Message.h>
#include <MessageQueue.h>
#include <Messenger.h>
#include <MessageRunner.h>
#include "BMADefs.h"
#include "BMAProperty.h"
#include "BMAMenu.h"
#include "BMAIcon.h"


enum
{
  OPEN_MENU = 'Lopm',
};


BMAIcon::BMAIcon(void)
  : BView(BRect(0, 0, 15, 15),
	  BMA_APP_SIGNATURE,
	  B_FOLLOW_NONE,
	  B_WILL_DRAW)
  , mMenuTrigger(NULL)
{
  app_info appInfo;
  BAppFileInfo fileInfo;
  BFile file;

  this->SetViewColor
    (::tint_color(::ui_color(B_PANEL_BACKGROUND_COLOR), B_DARKEN_1_TINT));
  this->SetDrawingMode(B_OP_OVER);

  mIcon = new BBitmap(BRect(0, 0, 15, 15), B_CMAP8);

  be_app->GetAppInfo(&appInfo);
  file.SetTo(&appInfo.ref, B_READ_ONLY);
  fileInfo.SetTo(&file);
  fileInfo.GetIcon(mIcon, B_MINI_ICON);

  mProperty = new BMAProperty();
  mProperty->SetNotifier(this);
  mProperty->Run();
}


BMAIcon::BMAIcon(BMessage *msg)
  : BView(msg)
  , mMenuTrigger(NULL)
{
  this->Unarchive(msg);

  this->SetViewColor
    (::tint_color(::ui_color(B_PANEL_BACKGROUND_COLOR), B_DARKEN_1_TINT));

  mProperty = new BMAProperty();
  mProperty->SetNotifier(this);
  mProperty->Run();
}


BMAIcon::~BMAIcon(void)
{
  delete mIcon;
  
  mProperty->Lock();
  mProperty->Quit();

  this->ResetMenuTrigger();
}


BArchivable *
BMAIcon::Instantiate(BMessage *archive)
{
  if (validate_instantiation(archive, "BMAIcon"))
    return new BMAIcon(archive);

  return NULL;
}


status_t
BMAIcon::Archive(BMessage *archive, bool deep) const
{
  BMessage bitmapArchive;

  archive->AddString("add_on", BMA_APP_SIGNATURE);

  if (deep)
    {
      mIcon->Archive(&bitmapArchive);
      archive->AddMessage("icon", &bitmapArchive);
    }

  BView::Archive(archive, deep);

  return B_OK;
}


void
BMAIcon::Unarchive(BMessage *archive)
{
  BMessage bitmapArchive;

  archive->FindMessage("icon", &bitmapArchive);
  mIcon = new BBitmap(&bitmapArchive);
}


void
BMAIcon::ResetMenuTrigger(void)
{
  delete mMenuTrigger;
  mMenuTrigger = NULL;
}


BMAProperty *
BMAIcon::Property(void) const
{
  return mProperty;
}


void
BMAIcon::Draw(BRect udpateRect)
{
  this->DrawBitmap(mIcon);
}


void
BMAIcon::OpenMenu(BPoint point)
{
  BPopUpMenu *menu;

  this->ConvertToScreen(&point);
  menu = new BMAMenu(this);
  menu->Go(point, true, true, BRect(-5, -5, 5, 5).OffsetBySelf(point));
  delete menu;
}


void
BMAIcon::MouseDown(BPoint point)
{
  int32 buttons, clicks;

  this->Looper()->CurrentMessage()->FindInt32("buttons", &buttons);
  this->Looper()->CurrentMessage()->FindInt32("clicks", &clicks);

  if (buttons & B_SECONDARY_MOUSE_BUTTON)
    {
      this->OpenMenu(point);
      return;
    }

  if (clicks >= 2)
    {
      mProperty->PostMessage(BMA_CHECK_NOW);
      return;
    }

  mMousePoint = point;
  this->SetMouseEventMask(B_POINTER_EVENTS);
  BMessage msg(OPEN_MENU);
  mMenuTrigger = new BMessageRunner(BMessenger(this),
				    &msg,
				    BMA_INTERVAL_TILL_MENU_OPENED,
				    1);
}


void
BMAIcon::MouseUp(BPoint point)
{
  this->ResetMenuTrigger();
}


void
BMAIcon::MouseMoved(BPoint point, uint32 transit, const BMessage *msg)
{
  this->Looper()->CurrentMessage()->FindPoint("be:view_where", &point);

  if (point != mMousePoint)
    this->ResetMenuTrigger();
}


void
BMAIcon::MessageReceived(BMessage *msg)
{
  switch (msg->what)
    {
    default:
      BView::MessageReceived(msg);
      break;

    case OPEN_MENU:
      this->ResetMenuTrigger();
      this->OpenMenu(mMousePoint);
      break;

    case BMA_BEGIN_CHECK:
      this->SetViewColor(255, 0, 0);
      this->Invalidate();
      break;

    case BMA_END_CHECK:
      this->SetViewColor
	(::tint_color(::ui_color(B_PANEL_BACKGROUND_COLOR), B_DARKEN_1_TINT));
      this->Invalidate();
      break;
    }
}

